// import 'package:cached_network_image/cached_network_image.dart';
import 'package:bytedesk_kefu/model/message.dart';
import 'package:bytedesk_kefu/ui/widget/bubble.dart';
import 'package:bytedesk_kefu/ui/widget/bubble_menu.dart';
import 'package:bytedesk_kefu/ui/widget/photo_view_wrapper.dart';
import 'package:bytedesk_kefu/util/bytedesk_constants.dart';
import 'package:bytedesk_kefu/util/bytedesk_events.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
// import 'package:fluttertoast/fluttertoast.dart';

class MessageWidget extends StatelessWidget {
  //
  final Message message;
  final themeColor = Color(0xfff5a623);
  final primaryColor = Color(0xff203152);
  final greyColor = Color(0xffaeaeae);
  final greyColor2 = Color(0xffE8E8E8);

  final AnimationController animationController;

  MessageWidget({this.message, this.animationController});

  @override
  Widget build(BuildContext context) {
    // 头像组件
    return message.isSend == 1
        ? _buildSendWidget(context)
        : _buildReceiveWidget(context);
  }

  // 发送消息widget
  Widget _buildSendWidget(BuildContext context) {
    double tWidth = MediaQuery.of(context).size.width - 160;
    return new Container(
        margin: EdgeInsets.only(top: 8.0, left: 8.0),
        padding: EdgeInsets.all(8.0),
        child: Column(
          children: <Widget>[
            // 时间戳
            _buildTimestampWidget(),
            // 消息内容
            Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Expanded(
                    child: Row(
                  children: <Widget>[
                    Expanded(flex: 1, child: Container()),
                    // Text(message.status ?? ''), // TODO：发送状态
                    Column(
                      // Column被Expanded包裹起来，使其内部文本可自动换行
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: <Widget>[
                        FLBubble(
                            from: FLBubbleFrom.right,
                            backgroundColor: Color.fromRGBO(160, 231, 90, 1),
                            child: Container(
                              constraints: BoxConstraints(maxWidth: tWidth),
                              padding: EdgeInsets.symmetric(
                                  horizontal: 5, vertical: 8),
                              child: _buildSendMenuWidget(context, message),
                            )),
                      ],
                    )
                  ],
                )),
                // 头像
                _buildAvatarWidget()
              ],
            ),
          ],
        ));
  }

  // 点击消息体菜单
  Widget _buildSendMenuWidget(BuildContext context, Message message) {
    return FLBubbleMenuWidget(
      interaction: FLBubbleMenuInteraction.tap,
      child: _buildSendContent(context, message),
      itemBuilder: (BuildContext context) {
        return [
          FLBubbleMenuItem(
            text: '复制',
            value: 'copy',
          ),
          FLBubbleMenuItem(
            text: '删除',
            value: 'delete',
          ),
          // TODO: 消息撤回
          // TODO: 消息回复
        ];
      },
      onSelected: (value) {
        print('send menu $value');
        // 删除消息
        if (value == 'copy') {
          /// 把文本复制进入粘贴板
          if (message.type == BytedeskConstants.MESSAGE_TYPE_TEXT) {
            Clipboard.setData(ClipboardData(text: message.content));
          } else if (message.type == BytedeskConstants.MESSAGE_TYPE_IMAGE) {
            Clipboard.setData(ClipboardData(text: message.imageUrl));
          } else {
            Clipboard.setData(ClipboardData(text: message.content));
          }
          // Toast
          Fluttertoast.showToast(
            msg: '复制成功',
            gravity: ToastGravity.BOTTOM,
            timeInSecForIosWeb: 30,
            backgroundColor: Colors.red,
            textColor: Colors.white,
            fontSize: 16.0);
        } else if (value == 'delete') {
          bytedeskEventBus.fire(DeleteMessageEventBus(message));
        }
      },
      onCancelled: () {
        print('cancel');
      },
    );
  }

  // 发送消息体
  Widget _buildSendContent(BuildContext context, Message message) {
    //
    if (message.type == BytedeskConstants.MESSAGE_TYPE_IMAGE) {
      return InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => PhotoViewWrapper(
                imageProvider: NetworkImage(
                  message.imageUrl,
                ),
                loadingBuilder: (context, event) {
                  if (event == null) {
                    return const Center(
                      child: Text("Loading"),
                    );
                  }
                  final value =
                      event.cumulativeBytesLoaded / event.expectedTotalBytes;
                  final percentage = (100 * value).floor();
                  return Center(
                    child: Text("$percentage%"),
                  );
                },
              ),
            ),
          );
        },
        child: SizedBox(
          width: 100,
          child: CachedNetworkImage(
            imageUrl: message.imageUrl,
            // placeholder: (context, url) => CircularProgressIndicator(),
            errorWidget: (context, url, error) => Icon(Icons.error),
          ),
        ),
        // child: Image.network(
        //   message.imageUrl,
        //   width: 100,
        // ),
      );
    } else {
      return Text(
        message.content ?? '',
        textAlign: TextAlign.right,
        softWrap: true,
        style: TextStyle(color: Colors.black, fontSize: 16.0),
      );
    }
  }

  // 接收消息widget
  Widget _buildReceiveWidget(BuildContext context) {
    double tWidth = MediaQuery.of(context).size.width - 160;
    return new Container(
        margin: EdgeInsets.only(top: 8.0, right: 8.0),
        padding: EdgeInsets.all(8.0),
        child: Column(children: <Widget>[
          // 时间戳
          _buildTimestampWidget(),
          // 消息
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              // 头像
              _buildAvatarWidget(),
              //
              FLBubble(
                  from: FLBubbleFrom.left,
                  backgroundColor: Colors.white,
                  child: Container(
                    constraints: BoxConstraints(maxWidth: tWidth),
                    padding: EdgeInsets.symmetric(horizontal: 5, vertical: 8),
                    child: _buildReceiveMenuWidget(context, message),
                  ))
            ],
          ),
        ]));
  }

  // 点击消息体菜单
  Widget _buildReceiveMenuWidget(BuildContext context, Message message) {
    return FLBubbleMenuWidget(
      interaction: FLBubbleMenuInteraction.tap,
      child: _buildReceivedContent(context, message),
      itemBuilder: (BuildContext context) {
        return [
          FLBubbleMenuItem(
            text: '复制',
            value: 'copy',
          ),
          FLBubbleMenuItem(
            text: '删除',
            value: 'delete',
          ),
          // TODO: 消息撤回
          // TODO: 消息回复
        ];
      },
      onSelected: (value) {
        print('send menu $value');
        // 删除消息
        if (value == 'copy') {
          /// 把文本复制进入粘贴板
          if (message.type == BytedeskConstants.MESSAGE_TYPE_TEXT) {
            Clipboard.setData(ClipboardData(text: message.content));
          } else if (message.type == BytedeskConstants.MESSAGE_TYPE_IMAGE) {
            Clipboard.setData(ClipboardData(text: message.imageUrl));
          } else {
            Clipboard.setData(ClipboardData(text: message.content));
          }
          // Toast
          Fluttertoast.showToast(
            msg: '复制成功',
            gravity: ToastGravity.BOTTOM,
            timeInSecForIosWeb: 30,
            backgroundColor: Colors.red,
            textColor: Colors.white,
            fontSize: 16.0);
        } else if (value == 'delete') {
          bytedeskEventBus.fire(DeleteMessageEventBus(message));
        }
      },
      onCancelled: () {
        print('cancel');
      },
    );
  }

  // 接收消息体
  Widget _buildReceivedContent(BuildContext context, Message message) {
    //
    if (message.type == BytedeskConstants.MESSAGE_TYPE_TEXT) {
      return Text(
        message.content ?? '',
        textAlign: TextAlign.left,
        softWrap: true,
        style: TextStyle(
          color: Colors.black,
          fontSize: 16.0,
        ),
      );
    } else if (message.type == BytedeskConstants.MESSAGE_TYPE_IMAGE) {
      return InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => PhotoViewWrapper(
                imageProvider: NetworkImage(
                  message.imageUrl,
                ),
                loadingBuilder: (context, event) {
                  if (event == null) {
                    return const Center(
                      child: Text("Loading"),
                    );
                  }
                  final value =
                      event.cumulativeBytesLoaded / event.expectedTotalBytes;
                  final percentage = (100 * value).floor();
                  return Center(
                    child: Text("$percentage%"),
                  );
                },
              ),
            ),
          );
        },
        child: SizedBox(
          width: 100,
          child: CachedNetworkImage(
            imageUrl: message.imageUrl,
            // placeholder: (context, url) => CircularProgressIndicator(),
            errorWidget: (context, url, error) => Icon(Icons.error),
          ),
        ),
      );
    } else if (message.type == BytedeskConstants.MESSAGE_TYPE_ROBOT) {
      return Column(
        children: <Widget>[
          Text(
            message.content ?? '',
            textAlign: TextAlign.left,
            softWrap: true,
            style: TextStyle(color: Colors.black, fontSize: 16.0),
          ),
          ListView.builder(
            // 如果滚动视图在滚动方向无界约束，那么shrinkWrap必须为true
            shrinkWrap: true,
            // 禁用ListView滑动，使用外层的ScrollView滑动
            physics: const NeverScrollableScrollPhysics(),
            itemCount: message.answers == null ? 0 : message.answers.length,
            itemBuilder: (_, index) {
              //
              var answer = message.answers[index];
              return new InkWell(
                  child: new Text(answer.question),
                  onTap: () => {print('object:' + answer.question)});
            },
          )
        ],
      );
    } else {
      return Text(
        message.content ?? '',
        textAlign: TextAlign.left,
        softWrap: true,
        style: TextStyle(color: Colors.black, fontSize: 16.0),
      );
    }
  }

  // 时间戳
  Widget _buildTimestampWidget() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Container(
          padding: EdgeInsets.only(bottom: 5, top: 5),
          child: Text(
            message.timestamp ?? '',
            textAlign: TextAlign.center,
            style: TextStyle(fontSize: 10.0),
          ),
        )
      ],
    );
  }

  // 头像
  Widget _buildAvatarWidget() {
    return SizedBox(
      width: 35,
      height: 35,
      child: CachedNetworkImage(
        imageUrl: message.avatar,
        // placeholder: (context, url) => CircularProgressIndicator(),
        errorWidget: (context, url, error) => Icon(Icons.error),
      ),
    );
  }
}
